
/*

  Read a data file in the form generated by the propagator program,
  and write a `source' file for calculating S_1 propagators.

*/

#include <stdio.h>
#include "datahdr.h"
#include "strmhdr.h"

#define S_SLICE 0
#define S_POINT 1

static char *usage        = "USAGE: dsource <state> -s <infile> <slice>\n       dsource <state> -p <spin/colour>\n" ;

void get_params (state, type, file1, slice, sc, argc, argv)
char **state, **file1 ;
int  *type, *slice, *sc ;
int argc ;
char *argv [] ;
{
  if (argc < 3)
  {
    fprintf (stderr, usage) ;
    exit (1) ;
  }
  *slice = 0 ;
  *state = argv [1] ;

  if      (! strcmp (argv [2], "-s"))
  {
    *type = S_SLICE ;

    if (argc != 5)
    {
      fprintf (stderr, usage) ;
      exit (1) ;
    }

    if (! strcmp (argv [3], "-"))
      *file1 = NULL ;
    else
      *file1 = argv [3] ;

    *slice = atoi (argv [4]) ;
  }
  else if (! strcmp (argv [2], "-p"))
  {
    *type = S_POINT ;

    if (argc != 4)
    {
      fprintf (stderr, usage) ;
      exit (1) ;
    }

    *sc = atoi (argv [3]) ;
  }
  else
  {
    fprintf (stderr, usage) ;
    exit (1) ;
  }
}

void main (argc, argv)
int argc ;
char *argv [] ;
{
  char *state, *file1 ;
  int type, slice, sc ;
  DATA *datas, *data1 ;
  DATA *data2 ;
  int   params [NUM_STT_PARAMS + NUM_DAT_PARAMS] ;
  float coeffs [NUM_STT_COEFFS + NUM_DAT_COEFFS] ;
  int l_time, l_space ;

  get_params  (&state, &type, &file1, &slice, &sc, argc, argv) ;

  datas = data_open_read (state) ;
  read_header     (datas) ;
  read_coeffs     (datas, coeffs) ;
  read_params     (datas, params) ;
  data_close_read (datas) ;

  data2 = data_dopen_write (1) ;
  {
    write_header (data2) ;
    write_coeffs (data2, 0, NULL) ;
    write_params (data2, 1, &slice) ;
  }
  write_field_begin (data2) ;
  fprintf (stderr, "%d\n", params [STT_L_T   ]) ;

  l_time  = params [STT_L_T] ;
  l_space = params [STT_L_X] * params [STT_L_Y] * params [STT_L_Z] ;

  switch (type)
  {
    case S_SLICE :
    {
      data1 = data_open_read (file1) ;
      read_header (data1) ;
      read_coeffs (data1, coeffs + NUM_STT_COEFFS) ;
      read_params (data1, params + NUM_STT_PARAMS) ;

      fprintf (stderr, "%d\n", params [NUM_STT_PARAMS + DAT_SC    ]) ;
      fprintf (stderr, "%f\n", coeffs [NUM_STT_COEFFS + DAT_K_FERM]) ;
      fprintf (stderr, "\n") ;

      read_field_begin  (data1) ;
      get_slice (data1, data2, l_time, l_space, slice) ;

      data_close_read  (data1) ;
    }
    break ;
    case S_POINT :
    {
      pnt_slice (data2, l_space, sc) ;
    }
    break ;
  }

  write_field_end  (data2) ;
  data_close_write (data2) ;
}

